@@ -41,12 +41,23 @@
 #' @param daysPriorHistory The number of days of prior history observed in
 #' the database required for an individual to start contributing time in
 #' a cohort.
+#' @param requirementInteractions If TRUE, cohorts will be created for
+#' all combinations of ageGroup, sex, and daysPriorHistory. If FALSE, only the
+#' first value specified for the other factors will be used. Consequently,
+#' order of values matters when requirementInteractions is FALSE.
 #' @param strataTable A cohort table in the cdm reference to use
 #' to limit cohort entry and exit (with individuals only contributing to a
 #' cohort when they are contributing to the cohort in the strata table).
 #' @param strataCohortId The cohort definition id for the cohort of interest
 #'  in the strata table. If strataTable is specified, a single strataCohortId
 #'  must also be specified.
+#' @param strataRequirementsAtEntry If TRUE, individuals will only be
+#' included if they satisfy time varying requirements on their strata cohort
+#' start date. If FALSE, individuals will enter the denominator cohort once
+#' they satisfy requirements.
+#' @param closedCohort If TRUE, a closed cohort will be defined where only
+#' those individuals satisfying eligibility criteria on the start date
+#' given in cohortDateRange are included.
 #' @param temporary If TRUE, temporary tables will be used throughout. If
 #' FALSE, permanent tables will be created in the write_schema of the cdm
 #' using the write_prefix (if specified). Note existing permanent tables in
@@ -71,18 +82,26 @@ generateDenominatorCohortSet <- function(cdm,
                                          ageGroup = list(c(0, 150)),
                                          sex = "Both",
                                          daysPriorHistory = 0,
+                                         requirementInteractions = TRUE,
                                          strataTable = NULL,
                                          strataCohortId = NULL,
+                                         strataRequirementsAtEntry = TRUE,
+                                         closedCohort = FALSE,
                                          temporary = TRUE) {
-
- startCollect <- Sys.time()
+  startCollect <- Sys.time()
 
   checkInputGenerateDCS(
-    cdm, name,
-    cohortDateRange,
-    ageGroup, sex, daysPriorHistory,
-    strataTable, strataCohortId,
-    temporary
+    cdm = cdm,
+    name = name,
+    cohortDateRange = cohortDateRange,
+    ageGroup = ageGroup,
+    sex = sex,
+    daysPriorHistory = daysPriorHistory,
+    requirementInteractions = requirementInteractions,
+    strataTable = strataTable,
+    strataCohortId = strataCohortId,
+    closedCohort = closedCohort,
+    temporary = temporary
   )
 
   # add broadest possible age group if no age strata were given
@@ -112,29 +131,28 @@ generateDenominatorCohortSet <- function(cdm,
     endDate <- cohortDateRange[2]
   }
 
-  # summarise combinations of inputs
+  # define cohorts to generate
   ageGrDf <- data.frame(do.call(rbind, ageGroup)) %>%
     dplyr::mutate(age_group = paste0(.data$X1, ";", .data$X2))
-  popSpecs <- tidyr::expand_grid(
-    age_group = ageGrDf$age_group,
-    sex = .env$sex,
-    days_prior_history = .env$daysPriorHistory,
-    start_date = .env$startDate,
-    end_date = .env$endDate
-  )
-  popSpecs <- popSpecs %>%
-    tidyr::separate(.data$age_group,
-      c("min_age", "max_age"),
-      remove = FALSE
-    ) %>%
-    dplyr::mutate(min_age = as.numeric(.data$min_age)) %>%
-    dplyr::mutate(max_age = as.numeric(.data$max_age)) %>%
-    dplyr::mutate(cohort_definition_id = dplyr::row_number())
+
+  popSpecs <- buildPopSpecs(
+    ageGrDf = ageGrDf,
+    sex = sex,
+    daysPriorHistory = daysPriorHistory,
+    requirementInteractions = requirementInteractions
+  ) %>%
+    dplyr::mutate(
+      min_age = as.numeric(.data$min_age),
+      max_age = as.numeric(.data$max_age),
+      start_date = .env$startDate,
+      end_date = .env$endDate,
+      cohort_definition_id = dplyr::row_number()
+    )
 
   # get the overall contributing population (without stratification)
   # we need to the output the corresponding dates when getting the denominator
 
-  if(isTRUE(temporary)){
+  if (isTRUE(temporary)) {
     tablePrefix <- NULL
   } else {
     tablePrefix <- paste0(attr(cdm, "write_prefix"), name)
@@ -157,7 +175,7 @@ generateDenominatorCohortSet <- function(cdm,
     dplyr::pull()
 
   # build each of the cohorts of interest
-    message("Creating denominator cohorts")
+  message("Creating denominator cohorts")
 
   if (denominatorPopulationNrows == 0) {
     message("- No people found for any denominator population")
@@ -206,11 +224,10 @@ generateDenominatorCohortSet <- function(cdm,
 
     # count dropped for sex criteria
     for (i in seq_along(dpop$attrition)) {
-
       if (popSpecs$sex[[i]] == "Male") {
         dpop$attrition[[i]] <- recordAttrition(
           table = dpop$denominator_population %>%
-            dplyr::filter(sex=="Male"),
+            dplyr::filter(sex == "Male"),
           id = "person_id",
           reasonId = 8,
           reason = "Not Male",
@@ -220,7 +237,7 @@ generateDenominatorCohortSet <- function(cdm,
       if (popSpecs$sex[[i]] == "Female") {
         dpop$attrition[[i]] <- recordAttrition(
           table = dpop$denominator_population %>%
-            dplyr::filter(sex=="Female"),
+            dplyr::filter(sex == "Female"),
           id = "person_id",
           reasonId = 8,
           reason = "Not Female",
@@ -256,7 +273,8 @@ generateDenominatorCohortSet <- function(cdm,
             ),
           "subject_id" = "person_id"
         ) %>%
-        dplyr::select("subject_id", "cohort_start_date", "cohort_end_date") %>%
+        dplyr::select(dplyr::any_of(c("subject_id", "cohort_start_date", "cohort_end_date",
+                      "strata_start_date")))  %>%
         dplyr::filter(.data$cohort_start_date <= .data$cohort_end_date)
 
       dpop$attrition[[i]] <- recordAttrition(
@@ -267,11 +285,45 @@ generateDenominatorCohortSet <- function(cdm,
         existingAttrition = dpop$attrition[[i]]
       )
 
+      if(!is.null(strataTable) && isTRUE(strataRequirementsAtEntry)){
+        workingDpop <- workingDpop %>%
+          dplyr::filter(.data$cohort_start_date ==
+                          .data$strata_start_date) %>%
+          dplyr::select(dplyr::any_of(c("subject_id",
+                                        "cohort_start_date",
+                                        "cohort_end_date")))
+
+        dpop$attrition[[i]] <- recordAttrition(
+          table = workingDpop,
+          id = "subject_id",
+          reasonId = 11,
+          reason = glue::glue("Excluded after requiring requirements on strata cohort start date"),
+          existingAttrition = dpop$attrition[[i]]
+        )
+
+      }
+
+
+      if (isTRUE(closedCohort)) {
+        workingDpop <- workingDpop %>%
+          dplyr::filter(.data$cohort_start_date == .env$startDate)
+
+        dpop$attrition[[i]] <- recordAttrition(
+          table = workingDpop,
+          id = "subject_id",
+          reasonId = 12,
+          reason = glue::glue("Excluded after applying closed cohort restriction"),
+          existingAttrition = dpop$attrition[[i]]
+        )
+      }
+
       dpop$attrition[[i]]$cohort_definition_id <- popSpecs$cohort_definition_id[[i]]
       workingCount <- utils::tail(dpop$attrition[[i]]$number_records, 1)
       cohortCount[[i]] <- workingDpop %>%
-        dplyr::summarise(number_records = dplyr::n(),
-                         number_subjects = dplyr::n_distinct(.data$subject_id)) %>%
+        dplyr::summarise(
+          number_records = dplyr::n(),
+          number_subjects = dplyr::n_distinct(.data$subject_id)
+        ) %>%
         dplyr::collect() %>%
         dplyr::mutate(
           cohort_definition_id = popSpecs$cohort_definition_id[[i]]
@@ -289,31 +341,37 @@ generateDenominatorCohortSet <- function(cdm,
     }
     cli::cli_progress_done()
 
-    studyPops <- unionCohorts(cdm,
-                              studyPops,
-                              tablePrefix)
-
+    studyPops <- unionCohorts(
+      cdm,
+      studyPops,
+      tablePrefix
+    )
   }
 
 
   if (!is.null(tablePrefix)) {
     # drop the intermediate tables that may have been created
-    CDMConnector::dropTable(cdm = cdm,
-                            name = paste0(tablePrefix, "_cohorts"))
-    CDMConnector::dropTable(cdm = cdm,
-                            name = tidyselect::starts_with(paste0(tablePrefix, "_i_")))
+    CDMConnector::dropTable(
+      cdm = cdm,
+      name = paste0(tablePrefix, "_cohorts")
+    )
+    CDMConnector::dropTable(
+      cdm = cdm,
+      name = tidyselect::starts_with(paste0(tablePrefix, "_i_"))
+    )
   }
 
-    dur <- abs(as.numeric(Sys.time() - startCollect, units = "secs"))
-    message(glue::glue(
-      "Time taken to get cohorts: {floor(dur/60)} min and {dur %% 60 %/% 1} sec"
-    ))
+
+  if (length(studyPops) == 0) {
+    message("- No people found for any denominator population")
+    studyPops <- dplyr::tibble()
+  }
 
   # add strata info to settings
   if (is.null(strataTable)) {
     strataCohortId <- NA
     strataCohortName <- NA
-  } else{
+  } else {
     strataCohortName <- attr(cdm[[strataTable]], "cohort_set") %>%
       dplyr::filter(.data$cohort_definition_id == .env$strataCohortId) %>%
       dplyr::pull("cohort_name")
@@ -327,10 +385,14 @@ generateDenominatorCohortSet <- function(cdm,
   cohortSet <- popSpecs %>%
     dplyr::mutate(strata_cohort_definition_id = .env$strataCohortId) %>%
     dplyr::mutate(strata_cohort_name = .env$strataCohortName) %>%
-    dplyr::mutate(cohort_name = paste0("Denominator cohort ", .data$cohort_definition_id)) %>%
+    dplyr::mutate(closed_cohort = .env$closedCohort) %>%
+    dplyr::mutate(cohort_name = paste0("Denominator cohort ",
+                                       .data$cohort_definition_id)) %>%
     dplyr::select(!c("min_age", "max_age")) %>%
-    dplyr::relocate("cohort_definition_id")%>%
-    dplyr::relocate("cohort_name", .after = "cohort_definition_id")
+    dplyr::relocate("cohort_definition_id") %>%
+    dplyr::relocate("cohort_name", .after = "cohort_definition_id") %>%
+    dplyr::mutate(age_group =
+                    stringr::str_replace(.data$age_group, ";", " to "))
   attr(studyPops, "cohort_set") <- cohortSet
 
   cohortAttrition <- dplyr::bind_rows(dpop$attrition) %>%
@@ -338,34 +400,80 @@ generateDenominatorCohortSet <- function(cdm,
     dplyr::relocate("cohort_definition_id")
   attr(studyPops, "cohort_attrition") <- cohortAttrition
 
-  if(!is.null(tablePrefix)){
-  insertAttributes(cdm = cdm,
-                   tablePrefix = tablePrefix,
-                   name = name,
-                   cohortCount = cohortCount,
-                   cohortSet = cohortSet,
-                   cohortAttrition = cohortAttrition)
+  if (!is.null(tablePrefix)) {
+    insertAttributes(
+      cdm = cdm,
+      tablePrefix = tablePrefix,
+      name = name,
+      cohortCount = cohortCount,
+      cohortSet = cohortSet,
+      cohortAttrition = cohortAttrition
+    )
   }
 
-  class(studyPops) <- c("IncidencePrevalenceDenominator", "GeneratedCohortSet",
-                        class(studyPops))
+  class(studyPops) <- c(
+    "IncidencePrevalenceDenominator", "GeneratedCohortSet",
+    class(studyPops)
+  )
 
   cdm[[name]] <- studyPops
 
-
+  dur <- abs(as.numeric(Sys.time() - startCollect, units = "secs"))
+  message(glue::glue(
+    "Time taken to get cohorts: {floor(dur/60)} min and {dur %% 60 %/% 1} sec"
+  ))
 
   return(cdm)
 }
 
 
+# cohort specifications
+buildPopSpecs <- function(ageGrDf,
+                          sex,
+                          daysPriorHistory,
+                          requirementInteractions) {
+  if (isTRUE(requirementInteractions)) {
+    popSpecs <- tidyr::expand_grid(
+      age_group = ageGrDf$age_group,
+      sex = .env$sex,
+      days_prior_history = .env$daysPriorHistory
+    )
+  } else {
+    popSpecs <- dplyr::bind_rows(
+      dplyr::tibble(
+        age_group = ageGrDf$age_group,
+        sex = .env$sex[1],
+        days_prior_history = .env$daysPriorHistory[1]
+      ),
+      dplyr::tibble(
+        age_group = ageGrDf$age_group[1],
+        sex = .env$sex,
+        days_prior_history = .env$daysPriorHistory[1]
+      ),
+      dplyr::tibble(
+        age_group = ageGrDf$age_group[1],
+        sex = .env$sex[1],
+        days_prior_history = .env$daysPriorHistory
+      )
+    ) %>%
+      dplyr::distinct()
+  }
+
+  popSpecs <- popSpecs %>%
+    tidyr::separate(.data$age_group,
+      c("min_age", "max_age"),
+      remove = FALSE
+    )
+
+  return(popSpecs)
+}
+
 
 # union cohort tables
 # combine the set of separate cohort tables into a single table
 unionCohorts <- function(cdm,
                          studyPops,
-                         tablePrefix
-                         ){
-
+                         tablePrefix) {
   if (length(studyPops) != 0 && length(studyPops) < 10) {
     studyPops <- Reduce(dplyr::union_all, studyPops)
     if (is.null(tablePrefix)) {
@@ -434,32 +542,36 @@ unionCohorts <- function(cdm,
 
     # drop any batch permanent tables
     if (!is.null(tablePrefix)) {
-      CDMConnector::dropTable(cdm = cdm,
-                              name = tidyselect::starts_with(paste0(tablePrefix, "_batch_")))
+      CDMConnector::dropTable(
+        cdm = cdm,
+        name = tidyselect::starts_with(paste0(tablePrefix, "_batch_"))
+      )
     }
   }
 
   return(studyPops)
-
 }
 
 
 # insert attributes into database
 insertAttributes <- function(cdm, tablePrefix, name,
-                             cohortCount, cohortSet, cohortAttrition){
+                             cohortCount, cohortSet, cohortAttrition) {
   DBI::dbWriteTable(attr(cdm, "dbcon"),
-                    name = CDMConnector::inSchema(attr(cdm, "write_schema"), paste0(tablePrefix, "_count")),
-                    value = as.data.frame(cohortCount),
-                    overwrite = TRUE)
+    name = CDMConnector::inSchema(attr(cdm, "write_schema"),
+                                  paste0(tablePrefix, "_count")),
+    value = as.data.frame(cohortCount),
+    overwrite = TRUE
+  )
   DBI::dbWriteTable(attr(cdm, "dbcon"),
-                    name = CDMConnector::inSchema(attr(cdm, "write_schema"), paste0(tablePrefix, "_set")),
-                    value = as.data.frame(cohortSet),
-                    overwrite = TRUE)
+    name = CDMConnector::inSchema(attr(cdm, "write_schema"),
+                                  paste0(tablePrefix, "_set")),
+    value = as.data.frame(cohortSet),
+    overwrite = TRUE
+  )
   DBI::dbWriteTable(attr(cdm, "dbcon"),
-                    name = CDMConnector::inSchema(attr(cdm, "write_schema"), paste0(tablePrefix, "_attrition")),
-                    value = as.data.frame(cohortAttrition),
-                    overwrite = TRUE)
+    name = CDMConnector::inSchema(attr(cdm, "write_schema"),
+                                  paste0(tablePrefix, "_attrition")),
+    value = as.data.frame(cohortAttrition),
+    overwrite = TRUE
+  )
 }
-
-
-
